"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var template_1 = __importDefault(require("@babel/template"));
var utils_1 = require("./utils");
var const_1 = require("./const");
var visitor_1 = require("./visitor");
var getInitState = function () { return ({
    reactTransformed: false,
    reactSpecifiers: [],
    compNames: []
}); };
/**
 * Plugin Instance
 */
var RuiPlugin = /** @class */ (function () {
    function RuiPlugin(types, opts) {
        this.state = getInitState();
        this.types = types;
        this.opts = (0, utils_1.mergeOptions)(opts);
        this.getState = this.getState.bind(this);
        this.updateState = this.updateState.bind(this);
        this.bootstrap = this.bootstrap.bind(this);
        this.resetState = this.resetState.bind(this);
        this.overwriteComponents = this.overwriteComponents.bind(this);
    }
    RuiPlugin.prototype.getState = function () {
        return this.state;
    };
    RuiPlugin.prototype.updateState = function (key, payload) {
        if (typeof payload === 'function') {
            this.state[key] = payload(this.state[key]);
        }
        else {
            this.state[key] = payload;
        }
    };
    RuiPlugin.prototype.resetState = function () {
        this.state = getInitState();
    };
    RuiPlugin.prototype.bootstrap = function (path) {
        path.traverse(visitor_1.entryVisitor, { plugin: this });
    };
    /** Overwrite AST related to components by compNames */
    RuiPlugin.prototype.overwriteComponents = function (path) {
        var t = this.types;
        var compNames = this.state.compNames;
        var _a = this.opts, splitChunkByComp = _a.splitChunkByComp, chunkNames = _a.chunkNames, libraryName = _a.libraryName, _b = _a.libraryDir, pcLibraryDir = _b.pc, mLibraryDir = _b.mobile;
        path.traverse({
            ImportDeclaration: function (path) {
                // last importDeclaration
                if (!t.isImportDeclaration(path.getNextSibling())) {
                    compNames.forEach(function (name) {
                        var getAST = (0, template_1["default"])(const_1.componentTemplate, {
                            plugins: ['jsx'],
                            // 仅可保留表达式外的注释
                            preserveComments: true
                        });
                        var mName = "M".concat(name);
                        var pcName = "PC".concat(name);
                        // template不支持innerComment，手动添加ast
                        var mLibName = typeof mLibraryDir === 'function' ? mLibraryDir((0, utils_1.toKebabCase)(name)) : mLibraryDir;
                        var mImport = t.callExpression(t.identifier('import'), [
                            (0, utils_1.addChunkNameComment)(t.stringLiteral("".concat(libraryName, "/").concat(mLibName)), splitChunkByComp ? "".concat(chunkNames.mobile, "-").concat(name) : chunkNames.mobile),
                        ]);
                        var pcLibName = typeof pcLibraryDir === 'function' ? pcLibraryDir((0, utils_1.toKebabCase)(name)) : pcLibraryDir;
                        var pcImport = t.callExpression(t.identifier('import'), [
                            (0, utils_1.addChunkNameComment)(t.stringLiteral("".concat(libraryName, "/").concat(pcLibName)), splitChunkByComp ? "".concat(chunkNames.pc, "-").concat(name) : chunkNames.pc),
                        ]);
                        path.insertAfter(getAST({
                            USE_MOBILE: t.callExpression(t.identifier(const_1.ruiExtraSpecifier), []),
                            COMPONENT_NAME: t.identifier(name),
                            M_COMP_VAR: t.identifier(mName),
                            M_COMP_TAG: t.jsxIdentifier(mName),
                            M_IMPORT: mImport,
                            PC_COMP_VAR: t.identifier(pcName),
                            PC_COMP_TAG: t.jsxIdentifier(pcName),
                            PC_IMPORT: pcImport
                        }));
                    });
                }
            }
        });
    };
    return RuiPlugin;
}());
exports["default"] = RuiPlugin;
